package gs.config.v0

import java.time.Instant
import java.time.LocalDate
import scala.util.Try

/** Type class for types that can be parsed from raw string configuration.
  */
trait Configurable[A]:
  /** Parse some raw string value as the desired type.
    *
    * @param raw
    *   The raw value to parse.
    * @return
    *   The parsed value, or `None` if parsing failed.
    */
  def parse(raw: String): Option[A]

object Configurable:

  def apply[A](
    using
    C: Configurable[A]
  ): Configurable[A] = C

  given Configurable[String] with
    def parse(raw: String): Option[String] = Some(raw)

  given Configurable[Int] with
    def parse(raw: String): Option[Int] = Try(raw.toInt).toOption

  given Configurable[Long] with
    def parse(raw: String): Option[Long] = Try(raw.toLong).toOption

  given Configurable[Boolean] with
    def parse(raw: String): Option[Boolean] = Try(raw.toBoolean).toOption

  given Configurable[LocalDate] with

    def parse(raw: String): Option[LocalDate] =
      Try(LocalDate.parse(raw)).toOption

  given Configurable[Instant] with

    def parse(raw: String): Option[Instant] =
      Try(Instant.parse(raw)).toOption
